import argparse
from pathlib import Path

from .handlers import handle_template
from .types import exclude, extra
from .types import files
from .types import target
from .types import template
from .utils import get_random_secret_key


DEFAULT_TEMPLATE = Path(__file__).resolve().parent / 'template'


def start(context):
    project_name = context.get('project_name')
    target = context.pop('target')
    template = context.pop('template')
    files_patterns = context.pop('files')
    exclude_patterns = context.pop('exclude')
    context.setdefault(
        'secret_key',
        'django-insecure-' + get_random_secret_key()
    )

    handle_template(
        project_name,
        target,
        template,
        files_patterns,
        exclude_patterns,
        context
    )


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'project_name',
        nargs='?',
        default='config',
        help='Name of the project. Default: "config"'
    )
    parser.add_argument(
        'target',
        nargs='?',
        type=target,
        default='.',
        help='Optional destination directory. Default: "."'
    )
    parser.add_argument(
        '-t', '--template',
        default=DEFAULT_TEMPLATE,
        type=template,
        help='The path or URL to load the template from.'
    )
    parser.add_argument(
        '-f', '--files',
        action='extend',
        default=['*-tpl'],
        type=files,
        help='The file glob pattern(s) that should be rendered.'
             ' Separate multiple file patterns with spaces.'
             ' Default: ["*-tpl"].'
    )
    parser.add_argument(
        '-e', '--extra',
        type=extra,
        help='Extra configuration parameters that should be rendered.'
             ' Separate multiple parameters with spaces and key value pairs'
             ' with equal sign. Will be available from `extra` object.'
    )
    parser.add_argument(
        '-x', '--exclude',
        default=['.*', '__pycache__'],
        type=exclude,
        help='The directory glob pattern(s) that should be excluded.'
    )

    context = parser.parse_args().__dict__
    start(context)


if __name__ == '__main__':
    main()



import random
from jinja2 import Template


def get_content(file, context):
    template = Template(file.read_text())
    content = template.render(**context)
    eof = get_eof(file)
    return content + eof


def get_eof(file):
    ends = ['', '\n']
    size = file.stat().st_size
    return ends[size > 0]


def get_random_secret_key():
    chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
    return ''.join(random.choice(chars) for _ in range(50))


def get_subdir_name(url):
    url = url.replace('https://', '').split('/')
    repo = url[2]
    branch = url[4].replace('.zip', '')
    return f'{repo}-{branch}'


def is_url(template):
    template = str(template)
    schemes = ['http', 'https', 'ftp']

    if ':' not in template:
        return False

    scheme = template.split(':', 1)[0].lower()
    return scheme in schemes


def match(entity, patterns):
    return any(entity.match(pattern) for pattern in patterns)



import os
import shutil
from pathlib import Path
from urllib.request import urlretrieve
from zipfile import is_zipfile
from zipfile import ZipFile

from .utils import get_content
from .utils import get_subdir_name
from .utils import is_url
from .utils import match


def handle_dir(
    dir,
    dst,
    files_patterns,
    to_render,
    exclude_patterns,
    project_name
):
    if match(dir, exclude_patterns):
        return None

    if dir.match('project_name'):
        dst = dst.with_name(project_name)

    new = handle_tree(
        dir,
        dst,
        project_name,
        files_patterns
    )
    to_render.extend(new)


def handle_file(
    file,
    dst,
    patterns,
    to_render
):
    if match(file, patterns):
        dst = dst.with_name(dst.name.replace('-tpl', ''))
        to_render.append(dst)
    shutil.copyfile(file, dst)


def handle_template(
    project_name,
    target,
    template,
    files_patterns,
    exclude_patterns,
    context
):
    args = [
        template,
        target,
        project_name,
        files_patterns,
        exclude_patterns
    ]
    if is_url(template):
        files = handle_url(*args)
    elif is_zipfile(template):
        files = handle_zip(*args)
    else:
        files = handle_tree(*args)

    for file in files:
        content = get_content(file, context)
        file.write_text(content)


def handle_tree(
    path,
    target,
    project_name,
    files_patterns,
    exclude_patterns
):
    path = Path(path).resolve()
    to_render = []
    os.makedirs(target, exist_ok=True)

    for entity in path.iterdir():
        dst = target / entity.name

        if entity.is_file():
            handle_file(
                entity,
                dst,
                files_patterns,
                to_render
            )
        if entity.is_dir():
            handle_dir(
                entity,
                dst,
                files_patterns,
                to_render,
                exclude_patterns,
                project_name
            )

    return to_render


def handle_url(
    url,
    target,
    project_name,
    files_patterns,
    exclude_patterns
):
    content, _ = urlretrieve(url)

    with ZipFile(content) as archive:
        archive.extractall()

    # There is subdir in the GitHub repository zip archive
    # like this: django-start-tool-main -- '{repo}-{branch}'.
    # It is necessary to extract all the contents and delete this subdir.
    subdir_name = get_subdir_name(url)
    files = handle_tree(
        subdir_name,
        target,
        project_name,
        files_patterns,
        exclude_patterns
    )
    shutil.rmtree(subdir_name)
    return files


def handle_zip(
    path,
    target,
    project_name,
    files_patterns,
    exclude_patterns
):
    tmp = target / 'tmp'

    with ZipFile(path) as archive:
        archive.extractall(tmp)

    files = handle_tree(
        tmp,
        target,
        project_name,
        files_patterns,
        exclude_patterns
    )
    shutil.rmtree(tmp)
    return files



from pathlib import Path

from .utils import is_url


def extra(context):
    result = {}

    for item in context.split():
        key, value = item.split('=')
        result.update({key: value})

    return result


def files(files):
    return files.split()


def target(path):
    return Path(path).resolve()


def template(path):
    if is_url(path):
        return path

    path = Path(path).resolve()
    if path.exists():
        return path

    raise Exception(f'Template does not exists: {str(path)}')

def exclude(entry):
    return entry.split()
